@@ -2,8 +2,11 @@
 
 source ~/.bashrc
 
+COMMAND="$@"
+if [[ -z "$COMMAND" ]]; then COMMAND="rake test"; fi
+
 for VERSION in `rvm list strings | sort`; do
 	rvm $VERSION
 	echo $(ruby -v)
-	RUBYOPT=-rubygems rake test
+	RUBYOPT=-rubygems $COMMAND
 done
