@@ -1,22 +1,9 @@
 # -*- coding: utf-8 -*-
 #
 #--
-# Copyright (C) 2009-2012 Thomas Leitner <t_leitner@gmx.at>
+# Copyright (C) 2009-2013 Thomas Leitner <t_leitner@gmx.at>
 #
-# This file is part of kramdown.
-#
-# kramdown is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# This file is part of kramdown which is licensed under the MIT.
 #++
 #
 
@@ -61,6 +48,7 @@ module Kramdown
         super
         @footnote_counter = @footnote_start = @options[:footnote_nr]
         @footnotes = []
+        @footnotes_by_name = {}
         @toc = []
         @toc_code = nil
         @indent = 2
@@ -105,7 +93,7 @@ module Kramdown
         if el.options[:transparent]
           inner(el, indent)
         else
-          "#{' '*indent}<p#{html_attributes(el.attr)}>#{inner(el, indent)}</p>\n"
+          format_as_block_html(el.type, el.attr, inner(el, indent), indent)
         end
       end
 
@@ -140,7 +128,7 @@ module Kramdown
       end
 
       def convert_blockquote(el, indent)
-        "#{' '*indent}<blockquote#{html_attributes(el.attr)}>\n#{inner(el, indent)}#{' '*indent}</blockquote>\n"
+        format_as_indented_block_html(el.type, el.attr, inner(el, indent), indent)
       end
 
       def convert_header(el, indent)
@@ -150,11 +138,11 @@ module Kramdown
         end
         @toc << [el.options[:level], attr['id'], el.children] if attr['id'] && in_toc?(el)
         level = output_header_level(el.options[:level])
-        "#{' '*indent}<h#{level}#{html_attributes(attr)}>#{inner(el, indent)}</h#{level}>\n"
+        format_as_block_html("h#{level}", attr, inner(el, indent), indent)
       end
 
       def convert_hr(el, indent)
-        "#{' '*indent}<hr />\n"
+        "#{' '*indent}<hr#{html_attributes(el.attr)} />\n"
       end
 
       def convert_ul(el, indent)
@@ -162,7 +150,7 @@ module Kramdown
           @toc_code = [el.type, el.attr, (0..128).to_a.map{|a| rand(36).to_s(36)}.join]
           @toc_code.last
         else
-          "#{' '*indent}<#{el.type}#{html_attributes(el.attr)}>\n#{inner(el, indent)}#{' '*indent}</#{el.type}>\n"
+          format_as_indented_block_html(el.type, el.attr, inner(el, indent), indent)
         end
       end
       alias :convert_ol :convert_ul
@@ -181,7 +169,7 @@ module Kramdown
       alias :convert_dd :convert_li
 
       def convert_dt(el, indent)
-        "#{' '*indent}<dt#{html_attributes(el.attr)}>#{inner(el, indent)}</dt>\n"
+        format_as_block_html(el.type, el.attr, inner(el, indent), indent)
       end
 
       def convert_html_element(el, indent)
@@ -218,15 +206,12 @@ module Kramdown
       alias :convert_xml_pi :convert_xml_comment
 
       def convert_table(el, indent)
-        "#{' '*indent}<table#{html_attributes(el.attr)}>\n#{inner(el, indent)}#{' '*indent}</table>\n"
+        format_as_indented_block_html(el.type, el.attr, inner(el, indent), indent)
       end
-
-      def convert_thead(el, indent)
-        "#{' '*indent}<#{el.type}#{html_attributes(el.attr)}>\n#{inner(el, indent)}#{' '*indent}</#{el.type}>\n"
-      end
-      alias :convert_tbody :convert_thead
-      alias :convert_tfoot :convert_thead
-      alias :convert_tr  :convert_thead
+      alias :convert_thead :convert_table
+      alias :convert_tbody :convert_table
+      alias :convert_tfoot :convert_table
+      alias :convert_tr  :convert_table
 
       ENTITY_NBSP = ::Kramdown::Utils::Entities.entity('nbsp') # :nodoc:
 
@@ -239,7 +224,7 @@ module Kramdown
           attr = el.attr.dup
           attr['style'] = (attr.has_key?('style') ? "#{attr['style']}; ": '') << "text-align: #{alignment}"
         end
-        "#{' '*indent}<#{type}#{html_attributes(attr)}>#{res.empty? ? entity_to_str(ENTITY_NBSP) : res}</#{type}>\n"
+        format_as_block_html(type, attr, res.empty? ? entity_to_str(ENTITY_NBSP) : res, indent)
       end
 
       def convert_comment(el, indent)
@@ -262,7 +247,7 @@ module Kramdown
           attr['href'] = obfuscate('mailto') << ":" << obfuscate(mail_addr)
           res = obfuscate(res) if res == mail_addr
         end
-        "<a#{html_attributes(attr)}>#{res}</a>"
+        format_as_span_html(el.type, attr, res)
       end
 
       def convert_img(el, indent)
@@ -271,19 +256,26 @@ module Kramdown
 
       def convert_codespan(el, indent)
         lang = extract_code_language(el.attr)
-        if @coderay_enabled && lang
-          result = CodeRay.scan(el.value, lang.to_sym).html(:wrap => :span, :css => @options[:coderay_css]).chomp
-          "<code#{html_attributes(el.attr)}>#{result}</code>"
-        else
-          "<code#{html_attributes(el.attr)}>#{escape_html(el.value)}</code>"
-        end
+        result = if @coderay_enabled && lang
+                   CodeRay.scan(el.value, lang.to_sym).html(:wrap => :span, :css => @options[:coderay_css]).chomp
+                 else
+                   escape_html(el.value)
+                 end
+        format_as_span_html('code', el.attr, result)
       end
 
       def convert_footnote(el, indent)
-        number = @footnote_counter
-        @footnote_counter += 1
-        @footnotes << [el.options[:name], el.value]
-        "<sup id=\"fnref:#{el.options[:name]}\"><a href=\"#fn:#{el.options[:name]}\" rel=\"footnote\">#{number}</a></sup>"
+        repeat = ''
+        if (footnote = @footnotes_by_name[el.options[:name]])
+          number = footnote[2]
+          repeat = ":#{footnote[3] += 1}"
+        else
+          number = @footnote_counter
+          @footnote_counter += 1
+          @footnotes << [el.options[:name], el.value, number, 0]
+          @footnotes_by_name[el.options[:name]] = @footnotes.last
+        end
+        "<sup id=\"fnref:#{el.options[:name]}#{repeat}\"><a href=\"#fn:#{el.options[:name]}\" class=\"footnote\">#{number}</a></sup>"
       end
 
       def convert_raw(el, indent)
@@ -295,7 +287,7 @@ module Kramdown
       end
 
       def convert_em(el, indent)
-        "<#{el.type}#{html_attributes(el.attr)}>#{inner(el, indent)}</#{el.type}>"
+        format_as_span_html(el.type, el.attr, inner(el, indent))
       end
       alias :convert_strong :convert_em
 
@@ -323,12 +315,17 @@ module Kramdown
       def convert_math(el, indent)
         block = (el.options[:category] == :block)
         value = (el.value =~ /<|&/ ? "% <![CDATA[\n#{el.value} %]]>" : el.value)
-        "<script type=\"math/tex#{block ? '; mode=display' : ''}\">#{value}</script>#{block ? "\n" : ''}"
+        type = {:type => "math/tex#{block ? '; mode=display' : ''}"}
+        if block
+          format_as_block_html('script', type, value, indent)
+        else
+          format_as_span_html('script', type, value)
+        end
       end
 
       def convert_abbreviation(el, indent)
         title = @root.options[:abbrev_defs][el.value]
-        "<abbr#{!title.empty? ? html_attributes(:title => title) : ''}>#{el.value}</abbr>"
+        format_as_span_html("abbr", {:title => (title.empty? ? nil : title)}, el.value)
       end
 
       def convert_root(el, indent)
@@ -346,6 +343,22 @@ module Kramdown
         result
       end
 
+      # Format the given element as span HTML.
+      def format_as_span_html(name, attr, body)
+        "<#{name}#{html_attributes(attr)}>#{body}</#{name}>"
+      end
+
+      # Format the given element as block HTML.
+      def format_as_block_html(name, attr, body, indent)
+        "#{' '*indent}<#{name}#{html_attributes(attr)}>#{body}</#{name}>\n"
+      end
+
+      # Format the given element as block HTML with a newline after the start tag and indentation
+      # before the end tag.
+      def format_as_indented_block_html(name, attr, body, indent)
+        "#{' '*indent}<#{name}#{html_attributes(attr)}>\n#{body}#{' '*indent}</#{name}>\n"
+      end
+
       # Generate and return an element tree for the table of contents.
       def generate_toc_tree(toc, type, attr)
         sections = Element.new(type, nil, attr)
@@ -400,24 +413,31 @@ module Kramdown
         result
       end
 
+      FOOTNOTE_BACKLINK_FMT = "%s<a href=\"#fnref:%s\" class=\"reversefootnote\">%s</a>"
+
       # Return a HTML ordered list with the footnote content for the used footnotes.
       def footnote_content
         ol = Element.new(:ol)
         ol.attr['start'] = @footnote_start if @footnote_start != 1
-        @footnotes.each do |name, data|
+        @footnotes.each do |name, data, _, repeat|
           li = Element.new(:li, nil, {'id' => "fn:#{name}"})
           li.children = Marshal.load(Marshal.dump(data.children))
           ol.children << li
 
-          ref = Element.new(:raw, "<a href=\"#fnref:#{name}\" rel=\"reference\">&#8617;</a>")
           if li.children.last.type == :p
             para = li.children.last
+            insert_space = true
           else
             li.children << (para = Element.new(:p))
+            insert_space = false
+          end
+
+          para.children << Element.new(:raw, FOOTNOTE_BACKLINK_FMT % [insert_space ? ' ' : '', name, "&#8617;"])
+          (1..repeat).each do |index|
+            para.children << Element.new(:raw, FOOTNOTE_BACKLINK_FMT % [" ", "#{name}:#{index}", "&#8617;<sup>#{index+1}</sup>"])
           end
-          para.children << ref
         end
-        (ol.children.empty? ? '' : "<div class=\"footnotes\">\n#{convert(ol, 2)}</div>\n")
+        (ol.children.empty? ? '' : format_as_indented_block_html('div', {:class => "footnotes"}, convert(ol, 2), 0))
       end
 
     end
