@@ -1,22 +1,9 @@
 # -*- coding: utf-8 -*-
 #
 #--
-# Copyright (C) 2009-2012 Thomas Leitner <t_leitner@gmx.at>
+# Copyright (C) 2009-2013 Thomas Leitner <t_leitner@gmx.at>
 #
-# This file is part of kramdown.
-#
-# kramdown is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# This file is part of kramdown which is licensed under the MIT.
 #++
 #
 
@@ -117,7 +104,7 @@ module Kramdown
 
       def convert_header(el, opts)
         res = ''
-        res << "#{'#' * el.options[:level]} #{inner(el, opts)}"
+        res << "#{'#' * output_header_level(el.options[:level])} #{inner(el, opts)}"
         res << "   {##{el.attr['id']}}" if el.attr['id'] && !el.attr['id'].strip.empty?
         res << "\n"
       end
@@ -161,7 +148,7 @@ module Kramdown
       end
 
       def convert_dd(el, opts)
-        sym, width = ": ", (el.children.first.type == :codeblock ? 4 : 2)
+        sym, width = ": ", (el.children.first && el.children.first.type == :codeblock ? 4 : 2)
         if ial = ial_for_element(el)
           sym << ial << " "
         end
@@ -171,12 +158,13 @@ module Kramdown
         newlines = text.scan(/\n*\Z/).first
         first, *last = text.split(/\n/)
         last = last.map {|l| " "*width + l}.join("\n")
-        text = first + (last.empty? ? "" : "\n") + last + newlines
+        text = first.to_s + (last.empty? ? "" : "\n") + last + newlines
         text.chomp! if text =~ /\n\n\Z/ && opts[:next] && opts[:next].type == :dd
-        text << "\n" if text !~ /\n\n\Z/ && opts[:next] && opts[:next].type == :dt
-        if el.children.first.type == :p && !el.children.first.options[:transparent]
+        text << "\n" if (text !~ /\n\n\Z/ && opts[:next] && opts[:next].type == :dt)
+        text << "\n" if el.children.empty?
+        if el.children.first && el.children.first.type == :p && !el.children.first.options[:transparent]
           "\n#{sym}#{text}"
-        elsif el.children.first.type == :codeblock
+        elsif el.children.first && el.children.first.type == :codeblock
           "#{sym}\n    #{text}"
         else
           "#{sym}#{text}"
@@ -201,8 +189,9 @@ module Kramdown
           "<#{el.value}#{html_attributes(el.attr)}" << (!res.empty? || HTML_TAGS_WITH_BODY.include?(el.value) ? ">#{res}</#{el.value}>" : " />")
         else
           output = ''
-          output << "<#{el.value}#{html_attributes(el.attr)}"
-          output << " markdown=\"1\"" if markdown_attr
+          attr = el.attr.dup
+          attr['markdown'] = '1' if markdown_attr
+          output << "<#{el.value}#{html_attributes(attr)}"
           if !res.empty? && el.options[:content_model] != :block
             output << ">#{res}</#{el.value}>"
           elsif !res.empty?
@@ -377,7 +366,7 @@ module Kramdown
         res << "\n\n" if @linkrefs.size > 0
         @linkrefs.each_with_index do |el, i|
           title = el.attr['title']
-          res << "[#{i+1}]: #{el.attr['href']} #{title ? '"' + title.gsub(/"/, "&quot;") + '"' : ''}\n"
+          res << "[#{i+1}]: #{el.attr['href']}#{title ? ' "' + title.gsub(/"/, "&quot;") + '"' : ''}\n"
         end
         res
       end
