@@ -1,22 +1,9 @@
 # -*- coding: utf-8 -*-
 #
 #--
-# Copyright (C) 2009-2012 Thomas Leitner <t_leitner@gmx.at>
+# Copyright (C) 2009-2013 Thomas Leitner <t_leitner@gmx.at>
 #
-# This file is part of kramdown.
-#
-# kramdown is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# This file is part of kramdown which is licensed under the MIT.
 #++
 #
 
@@ -97,7 +84,7 @@ module Kramdown
 
       def convert_codeblock(el, opts)
         show_whitespace = el.attr['class'].to_s =~ /\bshow-whitespaces\b/
-        lang = el.attr['lang']
+        lang = extract_code_language(el.attr)
         if show_whitespace || lang
           options = []
           options << "showspaces=%s,showtabs=%s" % (show_whitespace ? ['true', 'true'] : ['false', 'false'])
@@ -117,7 +104,7 @@ module Kramdown
       end
 
       def convert_header(el, opts)
-        type = @options[:latex_headers][el.options[:level] - 1]
+        type = @options[:latex_headers][output_header_level(el.options[:level]) - 1]
         if ((id = el.attr['id']) ||
             (@options[:auto_ids] && (id = generate_id(el.options[:raw_text])))) && in_toc?(el)
           "\\#{type}{#{inner(el, opts)}}\\hypertarget{#{id}}{}\\label{#{id}}\n\n"
@@ -158,10 +145,10 @@ module Kramdown
       end
 
       def convert_html_element(el, opts)
-        if el.value == 'i'
-          "\\emph{#{inner(el, opts)}}"
-        elsif el.value == 'b'
+        if el.value == 'i' || el.value == 'em'
           "\\emph{#{inner(el, opts)}}"
+        elsif el.value == 'b' || el.value == 'strong'
+          "\\textbf{#{inner(el, opts)}}"
         else
           warning("Can't convert HTML element")
           ''
@@ -219,9 +206,9 @@ module Kramdown
       def convert_a(el, opts)
         url = el.attr['href']
         if url =~ /^#/
-          "\\hyperlink{#{url[1..-1]}}{#{inner(el, opts)}}"
+          "\\hyperlink{#{escape(url[1..-1])}}{#{inner(el, opts)}}"
         else
-          "\\href{#{url}}{#{inner(el, opts)}}"
+          "\\href{#{escape(url)}}{#{inner(el, opts)}}"
         end
       end
 
@@ -417,7 +404,7 @@ module Kramdown
         402 => ['\textflorin', 'mathcomp'],
         381 => ['\v{Z}'],
         382 => ['\v{z}'],
-        160 => ['\nolinebreak'],
+        160 => ['~'],
         161 => ['\textexclamdown'],
         163 => ['\pounds'],
         164 => ['\currency', 'wasysym'],
@@ -558,7 +545,13 @@ module Kramdown
       end
 
       def convert_abbreviation(el, opts)
-        el.value
+        @data[:packages] += %w[acronym]
+        "\\ac{#{normalize_abbreviation_key(el.value)}}"
+      end
+
+      # Normalize the abbreviation key so that it only contains allowed ASCII character
+      def normalize_abbreviation_key(key)
+        key.gsub(/\W/) {|m| m.unpack('H*').first}
       end
 
       # Wrap the +text+ inside a LaTeX environment of type +type+. The element +el+ is passed on to
