@@ -1,27 +1,15 @@
 # -*- coding: utf-8 -*-
 #
 #--
-# Copyright (C) 2009-2012 Thomas Leitner <t_leitner@gmx.at>
+# Copyright (C) 2009-2013 Thomas Leitner <t_leitner@gmx.at>
 #
-# This file is part of kramdown.
-#
-# kramdown is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# This file is part of kramdown which is licensed under the MIT.
 #++
 #
 
 require 'rexml/parsers/baseparser'
 require 'strscan'
+require 'kramdown/utils'
 
 module Kramdown
 
@@ -83,9 +71,10 @@ module Kramdown
         # Process the HTML start tag that has already be scanned/checked via @src.
         #
         # Does the common processing steps and then yields to the caller for further processing
-        # (first parameter is the created element, the second parameter is +true+ if the HTML
-        # element is already closed, ie. contains no body).
-        def handle_html_start_tag # :yields: el, closed
+        # (first parameter is the created element; the second parameter is +true+ if the HTML
+        # element is already closed, ie. contains no body; the third parameter specifies whether the
+        # body - and the end tag - need to be handled in case closed=false).
+        def handle_html_start_tag # :yields: el, closed, handle_body
           name = @src[1].downcase
           closed = !@src[4].nil?
           attrs = Utils::OrderedHash.new
@@ -100,9 +89,9 @@ module Kramdown
           end
           if name == 'script' || name == 'style'
             handle_raw_html_tag(name)
-            yield(el, true)
+            yield(el, false, false)
           else
-            yield(el, closed)
+            yield(el, closed, true)
           end
         end
 
@@ -383,13 +372,13 @@ module Kramdown
               if c.type == :text
                 mem << c.value
               elsif c.type == :entity
-                if RUBY_VERSION >= '1.9'
-                  mem << c.value.char.encode(@root.options[:encoding])
-                elsif [60, 62, 34, 38].include?(c.value.code_point)
+                if [60, 62, 34, 38].include?(c.value.code_point)
                   mem << c.value.code_point.chr
+                else
+                  mem << c.value.char
                 end
               elsif c.type == :smart_quote || c.type == :typographic_sym
-                mem << entity(c.value.to_s).char.encode(@root.options[:encoding])
+                mem << entity(c.value.to_s).char
               else
                 raise "Bug - please report"
               end
@@ -405,6 +394,10 @@ module Kramdown
               set_basics(el, :codespan)
             else
               set_basics(el, :codeblock)
+              if el.children.size == 1 && el.children.first.value == 'code'
+                value = (el.children.first.attr['class'] || '').scan(/\blanguage-\w+\b/).first
+                el.attr['class'] = "#{value} #{el.attr['class']}".rstrip if value
+              end
             end
             el.value = result.first.value
             el.children.clear
@@ -554,8 +547,8 @@ module Kramdown
           end
         end
 
-        tag_handler = lambda do |c, closed|
-          parse_raw_html(c, &tag_handler) if !closed
+        tag_handler = lambda do |c, closed, handle_body|
+          parse_raw_html(c, &tag_handler) if !closed && handle_body
         end
         parse_raw_html(@tree, &tag_handler)
 
