@@ -1,22 +1,9 @@
 # -*- coding: utf-8 -*-
 #
 #--
-# Copyright (C) 2009-2012 Thomas Leitner <t_leitner@gmx.at>
+# Copyright (C) 2009-2013 Thomas Leitner <t_leitner@gmx.at>
 #
-# This file is part of kramdown.
-#
-# kramdown is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# This file is part of kramdown which is licensed under the MIT.
 #++
 #
 
@@ -52,8 +39,11 @@ module Kramdown
               result = []
               strscan = StringScanner.new(child.value)
               while temp = strscan.scan_until(regexps.last)
-                temp << strscan.scan(/\W|^/)
-                abbr = strscan.scan(regexps.first)
+                abbr = strscan.scan(regexps.first) # begin of line case of abbr with \W char as first one
+                if abbr.nil?
+                  temp << strscan.scan(/\W|^/)
+                  abbr = strscan.scan(regexps.first)
+                end
                 result << Element.new(:text, temp) << Element.new(:abbreviation, abbr)
               end
               result << Element.new(:text, strscan.rest)
